/*
 *  Copyright 2019-2020 Zheng Jie
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package me.zhengjie.modules.system.repository;

import me.zhengjie.modules.system.domain.User;
import org.springframework.data.jpa.repository.JpaRepository;
import org.springframework.data.jpa.repository.JpaSpecificationExecutor;
import org.springframework.data.jpa.repository.Modifying;
import org.springframework.data.jpa.repository.Query;
import org.springframework.data.repository.query.Param;

import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * @author Zheng Jie
 * @date 2018-11-22
 */
public interface UserRepository extends JpaRepository<User, String>, JpaSpecificationExecutor<User> {

    /**
     * 根据用户名查询
     * @param username 用户名
     * @return /
     */
    User findByUsername(String username);

    /**
     * 根据邮箱查询
     * @param email 邮箱
     * @return /
     */
    User findByEmail(String email);

    /**
     * 根据手机号查询
     * @param phone 手机号
     * @return /
     */
    User findByPhone(String phone);

    /**
     * 修改密码
     * @param username 用户名
     * @param pass 密码
     * @param lastPasswordResetTime /
     */
    @Modifying
    @Query(value = "update sys_user set password = ?2 , pwd_reset_time = ?3 where username = ?1",nativeQuery = true)
    void updatePass(String username, String pass, Date lastPasswordResetTime);

    /**
     * 修改邮箱
     * @param username 用户名
     * @param email 邮箱
     */
    @Modifying
    @Query(value = "update sys_user set email = ?2 where username = ?1",nativeQuery = true)
    void updateEmail(String username, String email);

    /**
     * 根据角色查询用户
     * @param roleId /
     * @return /
     */
    @Query(value = "SELECT u.* FROM sys_user u, sys_users_roles r WHERE" +
            " u.user_id = r.user_id AND r.role_id = ?1", nativeQuery = true)
    List<User> findByRoleId(String roleId);

    /**
     * 根据角色查询用户
     * @param roleId /
     * @return /
     */
    @Query(value = "SELECT u.user_id as userId,u.username FROM sys_user u, sys_users_roles r WHERE" +
            " u.user_id = r.user_id AND r.role_id = ?1", nativeQuery = true)
    List<Map<String,Object>> findByRoleId2(String roleId);

    /**
     * 根据角色中的部门查询
     * @param deptId /
     * @return /
     */
    @Query(value = "SELECT u.* FROM sys_user u, sys_users_roles r, sys_roles_depts d WHERE " +
            "u.user_id = r.user_id AND r.role_id = d.role_id AND d.dept_id = ?1 group by u.user_id", nativeQuery = true)
    List<User> findByRoleDeptId(String deptId);

    /**
     * 根据菜单查询
     * @param id 菜单ID
     * @return /
     */
    @Query(value = "SELECT u.* FROM sys_user u, sys_users_roles ur, sys_roles_menus rm WHERE\n" +
            "u.user_id = ur.user_id AND ur.role_id = rm.role_id AND rm.menu_id = ?1 group by u.user_id", nativeQuery = true)
    List<User> findByMenuId(String id);

    /**
     * 根据Id删除
     * @param ids /
     */
    void deleteAllByIdIn(Set<String> ids);

    /**
     * 根据岗位查询
     * @param ids /
     * @return /
     */
    @Query(value = "SELECT count(1) FROM sys_user u, sys_users_jobs j WHERE u.user_id = j.user_id AND j.job_id IN ?1", nativeQuery = true)
    int countByJobs(Set<String> ids);

    /**
     * 根据部门查询
     * @param deptIds /
     * @return /
     */
    @Query(value = "SELECT count(1) FROM sys_user u WHERE u.enabled = 1 and u.dept_id IN ?1", nativeQuery = true)
    int countByDepts(Set<String> deptIds);

    /**
     * 根据部门查询人数
     * @param deptId /
     * @return /
     */
    @Query(value = "SELECT count(1) FROM sys_user u WHERE u.enabled = 1 and u.dept_id = ?1", nativeQuery = true)
    Long countByDeptId(String deptId);

    /**
     * 根据部门查询月内新增人数
     * @param deptId /
     * @return /
     */
    @Query(value = "SELECT count(1) FROM sys_user u WHERE u.enabled = 1 and u.dept_id = ?1 AND u.create_time BETWEEN SUBDATE(CURDATE(),INTERVAL 1 MONTH) AND CURDATE() ", nativeQuery = true)
    Long countMonthByDeptId(String deptId);

    /**
     * 根据角色查询
     * @param ids /
     * @return /
     */
    @Query(value = "SELECT count(1) FROM sys_user u, sys_users_roles r WHERE " +
            "u.user_id = r.user_id AND r.role_id in ?1", nativeQuery = true)
    int countByRoles(Set<Long> ids);

    @Query(value = "SELECT COUNT(1) FROM sys_user WHERE company_id = ?1",nativeQuery = true)
    Integer selecIsNotCompany(Long companyId);

    @Query(value = "SELECT DISTINCT full_name FROM dm_company WHERE company_id in (?1)",nativeQuery = true)
    List selectCompanyName(Long[] companyId);

    /**
     * 根据userId查询
     * @param dmUserId /
     * @return /
     */
    @Query(value = "SELECT u.* FROM sys_user u WHERE " +
            "u.dm_user_id = :dmUserId",nativeQuery = true)
    User getUserByDmUserId(@Param("dmUserId")String dmUserId);

    /**
     * 根据卡号查询账号
     * @param cardId /
     * @return /
     */
    @Query(value = "select su.username from sys_user as su LEFT JOIN dm_user du ON su.user_id = du.id where du.card_id=:cardId",nativeQuery = true)
    String getUsernameByCardId(@Param("cardId")String cardId);

    /**
     * 根据用户id查询用户信息
     * @param user_number  用户uid
     * @return Map
     */
    @Query(value = "select su.name from dm_user as su where  su.id = :user_number",nativeQuery = true)
    Map<String, Object> getUserByUserNumber(@Param("user_number") String user_number);

    /**
     * 根据人员状态查询员工工号
     * @param enabled
     * @return
     */
    @Query(value = "select dm.user_number as userID,dm.id as persion_id from sys_user sys inner join dm_user dm on sys.user_id = dm.id where if(:enabled != null,sys.enabled = :enabled,1=1) AND if(:userId IS NOT NULL ,dm.id = :userId,1=1)",nativeQuery = true)
    List<Map<String, Object>> findAllByEnable(@Param("enabled") Integer enabled,@Param("userId") String userId);

}
